program sigmadelta_dsf;
(* Plain old single-bit sigma-delta modulator with no dither and almost no chaos. *)
(* Input signal: offset, output: dsf file. *)

uses Crt, strutils;

const
	DSD:int32 = 128;
	seconds = 10; // 60; // desired audio length in seconds
	overslaan:int64 = 1048576; // 1048576 samples for settling
	offsetL:extended = 5000/(0.5*5644800); // offset left channel, 0.5 corresponds to 0 dB DSD.
(* This value should cause tones 5 kHz above and below fs/2 -> second-order intermodulation at 10 kHz *)
	offsetR:extended = 5000/(0.5*5644800); // offset right channel
	freqL:extended = 1000; // frequency sine wave left channel, Hz
	freqR:extended = 1000; // frequency sine wave right channel, Hz
	amplL:extended = 0; // amplitude sine wave left channel, 0.5 corresponds to 0 dB DSD
	amplR:extended = 0; // amplitude sine wave right channel, 0.5 corresponds to 0 dB DSD

(* Coefficients for procedure sigmadelta. *)
	c1 = 0.000648018179;
	c2 = 0.010137894357;
	c3 = 0.073366653460;
	c4 = 0.315614906588;
	c5 = 0.807231788708;
	i1 = 1;
	i2 = 1;
	i3 = 1;
	i4 = 1;
	i5 = 1;
	d1 = 0.0001220703125; (* Notch around 9925.5835257506 Hz *)
	d2 = 0.00048828125; (* Notch around 19848.7446018647 Hz *)
	pi:extended = 4*arctan(1);
	

var
	lengte: int64;
	fs: int32;
	xinL: extended;
	xinR: extended;

(* int1 to int5: integrator states of sigmadelta *)
	int1L: extended;
	int2L: extended;
	int3L: extended;
	int4L: extended;
	int5L: extended;
	sdoutL: array[0..32767] of integer; // output signal sigma-delta
	int1R: extended;
	int2R: extended;
	int3R: extended;
	int4R: extended;
	int5R: extended;
	sdoutR: array[0..32767] of integer; // output signal sigma-delta

	teller: int64; // counter
	teller2: int64;
	teller3: int64;
	blokteller: int64; // counter

	f: file;
(*	s: string; *)
	c: AnsiChar;
	a: uint64;
	b: uint32;

	pattern: uint32;

procedure sigmadelta(xin: extended; var uit: integer; teller: int64; var int1: extended;
		var int2: extended; var int3: extended; var int4: extended; var int5: extended);
var
	int1oud: extended;
	int2oud: extended;
	int3oud: extended;
	int4oud: extended;
	int5oud: extended;

begin
	if int5 > 0 then
		uit:=1
	else
		uit:=-1;
	int1oud := int1;
	int2oud := int2;
	int3oud := int3;
	int4oud := int4;
	int5oud := int5;
	int1:=xin-c1*uit-d1*int2oud+i1*int1oud;
	int2:=int1oud-c2*uit+i2*int2oud;
	int3:=int2oud-c3*uit-d2*int4oud+i3*int3oud;
	int4:=int3oud-c4*uit+i4*int4oud;
	int5:=int4oud-c5*uit+i5*int5oud;
end; // procedure sigmadelta


begin // main program
	fs := DSD * 44100; // sample rate in Hz
	lengte := 4096*8*round(seconds*fs/(4096*8)); // Number of samples rounded to fit in an integer number of 4096 byte blocks

	assign(f, 'SDMout.dsf');

	rewrite(f,1);//<------ changed to 1 byte block instead of 128 bytes that is the default.
	c:='D';
	blockwrite(f,c,sizeof(c));
	c:='S';
	blockwrite(f,c,sizeof(c));
	c:='D';
	blockwrite(f,c,sizeof(c));
	c:=' ';
	blockwrite(f,c,sizeof(c));
	a:=28;
	blockwrite(f,a,sizeof(a));
	a:= 2 * (lengte div 8) + 92; (* File size in bytes: 80 plus length datachunk, 80 plus 12 plus length data. Factor of two because it's stereo. *)
	blockwrite(f,a,sizeof(a));
	a:=0;
	blockwrite(f,a,sizeof(a));
	c:='f';
	blockwrite(f,c,sizeof(c));
	c:='m';
	blockwrite(f,c,sizeof(c));
	c:='t';
	blockwrite(f,c,sizeof(c));
	c:=' ';
	blockwrite(f,c,sizeof(c));
	a:=52;
	blockwrite(f,a,sizeof(a));
	b:=1;
	blockwrite(f,b,sizeof(b));
	b:=0;
	blockwrite(f,b,sizeof(b));
	b:=2;
	blockwrite(f,b,sizeof(b));
	b:=2;
	blockwrite(f,b,sizeof(b));
	b:=fs;
	blockwrite(f,b,sizeof(b));
	b:=1;
	blockwrite(f,b,sizeof(b));
	a:=lengte; (* samples per channel *)
	blockwrite(f,a,sizeof(a));
	b:=4096;
	blockwrite(f,b,sizeof(b));
	b:=0;
	blockwrite(f,b,sizeof(b));
	c:='d';
	blockwrite(f,c,sizeof(c));
	c:='a';
	blockwrite(f,c,sizeof(c));
	c:='t';
	blockwrite(f,c,sizeof(c));
	c:='a';
	blockwrite(f,c,sizeof(c));
	a:= 2 * (lengte div 8) + 12; (* Size of data chunk in bytes: number of data bytes + 12. Times two because it's stereo. *)
	blockwrite(f,a,sizeof(a));

	int1L:=0.0006779986543;
	int2L:=0;
	int3L:=0;
	int4L:=0;
	int5L:=0.00014553722;
	int1R:=0.00054321;
	int2R:=0;
	int3R:=0;
	int4R:=0;
	int5R:=0.00012987;
	blokteller := 0;
	teller:=1;
	while (teller <= lengte + overslaan) do
	begin
		xinL := c1*(offsetL + amplL*sin(2*pi*freqL*teller/fs)); 
		xinR := c1*(offsetR + amplR*sin(2*pi*freqR*teller/fs)); 
		sigmadelta(xinL, sdoutL[blokteller], teller, int1L, int2L, int3L, int4L, int5L);
		sigmadelta(xinR, sdoutR[blokteller], teller, int1R, int2R, int3R, int4R, int5R);
		teller:=teller+1;
		blokteller := blokteller + 1;
		if blokteller = 32768 then // write data to dsf file when 32768 bits per channel have been generated
		begin
			blokteller := 0;
			if teller >= overslaan then
			begin
				for teller2 := 0 to 1023 do
				begin
					pattern := 0;
					for teller3 := 0 to 31 do pattern := 2*pattern + (sdoutL[32*teller2+31-teller3] + 1) div 2;
					b:=pattern;
					blockwrite(f,b,sizeof(b));
				end;
				for teller2 := 0 to 1023 do
				begin
					pattern := 0;
					for teller3 := 0 to 31 do pattern := 2*pattern + (sdoutR[32*teller2+31-teller3] + 1) div 2;
					b:=pattern;
					blockwrite(f,b,sizeof(b));
				end;
			end;
		end;
	end;
	close(f);
end.
